from django.core.management.base import BaseCommand
from GaiaDBInterface.models import GaiaSource, CatalogFile
from astropy.coordinates import SkyCoord
import astropy_healpix as ah
import numpy as np
from datetime import datetime, timedelta

def current_time():
    return (datetime.now() + timedelta(hours=3)).strftime("%H:%M:%S")

def healpix():

    NSIDE=2048
    ORDER='ring'
    
    for catalog_file in CatalogFile.objects.all():
        print(f'[{current_time()}] Loading sources from the database...')
        sources = list(catalog_file.sources.all())
        print(f'[{current_time()}] Sources ready. Forming ra & dec arrays...')

        ra_list = np.array([source.ra for source in sources])
        dec_list = np.array([source.dec for source in sources])
        print(f'[{current_time()}] ra & dec arrays ready. Creating SkyCoord objects...')

        skycoord = SkyCoord(ra=ra_list, dec=dec_list, unit='deg', frame='fk5')
        print(f'[{current_time()}] SkyCoord objects ready. Calculating healpix indices...')

        healpix = ah.HEALPix(nside=NSIDE, order=ORDER, frame='fk5')
        healpix_indices = healpix.skycoord_to_healpix(skycoord)
        print(f'[{current_time()}] Indices ready. Assigning indices to source instances...')

        for source, healpix_index in zip(sources, healpix_indices):
            source.healpix_ring_index = healpix_index
        print(f'[{current_time()}] Instances ready. Bulk updating the database...')

        GaiaSource.objects.bulk_update(sources, ['healpix_ring_index'])
        print(f'[{current_time()}] Database updated, sources indexed successfully.')


class Command(BaseCommand):
    help = 'Index sources using healpix.'

    def handle(self, *args, **options):

        healpix()